/*
 * ESCC CONFIGURATION PROGRAM
 *
 * Heavily based on
 *
 * piconfig.c: pi-ifconfig program for the Ottawa Amateur Radio Club
 * PI and PI2 interface card. 
 * Copyright (c) 1994 John Paul Morrison (VE7JPM).
 *
 * Mods for ESCC card by i1epj
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge MA 02139, USA.
 */

#include <net/if.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/errno.h>
#include "gpcmescc.h"

#define DEBUG 0

int opt_V = 0;
int opt_h = 0;
int opt_v = 0;
int skfd = -1;

struct ifreq ifr;
struct gpcm_req gpcm;

char *clkmodes[] = {
	"internal",
	"external",
};


void usage(void)
{
	fprintf(stderr, "\nUsage: gpcmconfig [-v] [-V] interface\n");
	fprintf(stderr, "                  [speed <baud>] [clockmode <mode>]\n");
	fprintf(stderr, "                  [txdelay <time>] [persist <persist value>]\n");
	fprintf(stderr, "                  [slot <time>] [txtail <time>]\n\n");
	fprintf(stderr, " -v  be verbose (redisplay interface settings after set up)\n");
	fprintf(stderr, " -V  display version string and exit successfully\n\n");
	fprintf(stderr, "<mode> must be 0 or 1; <persist value> must be 0...255\n");
	fprintf(stderr, "<time> is in ms if followed by 'ms', else in 100 us units\n");
	fprintf(stderr, "e.g. 'txdelay 50ms' and 'txdelay 500' are both a 50 ms txdelay.\n\n");
	exit(-1);
}

void print_gpcm(void)
{
	int clkmode = gpcm.clockmode;

	printf("\nInterface %s:\n", ifr.ifr_name);
	printf("\tclkmode\t%5d (%s)\n", clkmode, clkmodes[clkmode]);
	switch (gpcm.clockmode) {
	case INTERNAL:
		printf("\tspeed\t%5d baud\n", gpcm.speed);
		break;
	case EXTERNAL:
		printf("\tspeed\tdoes not matter (externally set)\n");
		break;
	}
	printf
	    ("\ttxdelay\t%5d (%3.1f ms)\n\tpersist\t%5d (%3.1f%%)\n\tslotime\t%5d (%3.1f ms)\n\ttxtail\t%5d (%3.1f ms)\n",
	     gpcm.txdelay, (float) gpcm.txdelay / 10.0, gpcm.persist,
	     (float) gpcm.persist / 2.55, gpcm.slotime,
	     (float) gpcm.slotime / 10.0, gpcm.squeldelay, (float) gpcm.squeldelay / 10.0);

	printf("\tirq\t%5d\n", gpcm.irq);
	if (gpcm.dmachan)
		printf("\tdma\t%5d\n", gpcm.dmachan);
	else
		printf("\tdma\t none\n");
}


int main(int argc, char **argv)
{
	int ret = 0, i;
	char **spp, *cp;

	if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("socket");
		exit(-1);
	}

	if (argc < 2)
		usage();

	/* Find any options. */
	argv++;
	while (**argv == '-') {
		if (!strcmp(*argv, "-v"))
			opt_v = 1;
		if (!strcmp(*argv, "-V"))
			opt_V = 1;
		argv++;
		argc--;
	}

	if (opt_V) {
		fprintf(stderr, "GPCM ESCC driver configuration program version 1.0\n");
		exit(0);
	}

	spp = argv;
	strncpy(ifr.ifr_name, *spp++, IFNAMSIZ);
	memset(&gpcm, 0, sizeof(struct gpcm_req));
	ifr.ifr_data = (caddr_t) & gpcm;

	gpcm.cmd = SIOCGGPCMPARAM;	/* get current settings for device */
	if (ioctl(skfd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("ioctl");
		exit(-2);
	}
	if (*spp == (char *) NULL) {
		print_gpcm();
		(void) close(skfd);
		exit(0);
	}
	while (*spp != (char *) NULL) {
		if (!strcmp(*spp, "speed")) {
			if (*++spp == NULL)
				usage();
			gpcm.speed = atoi(*spp);
			spp++;
			continue;
		} else if (!strcmp(*spp, "clockmode")) {
			if (*++spp == NULL)
				usage();
			gpcm.clockmode = atoi(*spp);
			spp++;
			continue;
		} else if (!strcmp(*spp, "txdelay")) {
			if (*++spp == NULL)
				usage();
			if ((cp = strstr(*spp, "ms")) != NULL) {
				*cp = '\0';
				gpcm.txdelay = 10 * atoi(*spp);
			} else {
				gpcm.txdelay = atoi(*spp);
			}
			spp++;
			continue;
		} else if (!strcmp(*spp, "persist")) {
			if (*++spp == NULL)
				usage();
			gpcm.persist = atoi(*spp);
			spp++;
			continue;
		} else if (!strcmp(*spp, "slot")) {
			if (*++spp == NULL)
				usage();
			if ((cp = strstr(*spp, "ms")) != NULL) {
				*cp = '\0';
				gpcm.slotime = 10 * atoi(*spp);
			} else {
				gpcm.slotime = atoi(*spp);
			}
			spp++;
			continue;
		} else if (!strcmp(*spp, "txtail")) {
			if (*++spp == NULL)
				usage();
			if ((cp = strstr(*spp, "ms")) != NULL) {
				*cp = '\0';
				gpcm.squeldelay = 10 * atoi(*spp);
			} else {
				gpcm.squeldelay = atoi(*spp);
			}
			spp++;
			continue;
		} else {
			fprintf(stderr, "gpcmconfig error: unknown option '%s'\n", *spp);
			exit(-2);
		}
		spp++;
	}

	gpcm.cmd = SIOCSGPCMPARAM;
	if (ioctl(skfd, SIOCDEVPRIVATE, &ifr) < 0)
		perror("ioctl");

	if (opt_v)
		print_gpcm();


	/* Close the socket. */
	(void) close(skfd);

	return (ret);
}
